/* (C) Copyright International Business Machines Corporation 23 January */
/* 1990.  All Rights Reserved. */
/*  */
/* See the file USERAGREEMENT distributed with this software for full */
/* terms and conditions of use. */
/* File: o_scalar.c */
/* Author: David F. Bacon */
#ifndef lint
static char sccsinfo[] = "@(#)o_scalar.c	1.6 3/13/90";
#endif

#include "li.h"

predef_exception
cp_scalar(dst,src)
valcell *dst,src;
{
    *dst = src;			/* copy the scalar value */
    return(Normal);		/* and succeed */
}


/*ARGSUSED*/
void
fin_scalar(value, f_op, sched)
valcell value;
finalize_op f_op;
schedblock *sched;
{
  /* finalization of a scalar is a no-op */
}
