/* (C) Copyright International Business Machines Corporation 23 January */
/* 1990.  All Rights Reserved. */
/*  */
/* See the file USERAGREEMENT distributed with this software for full */
/* terms and conditions of use. */
#ifndef lint
static char sccsinfo[] = "@(#)fakemalloc.c	1.2 3/13/90";
#endif

int
malloc_debug(level)
int level;
{
    if (level > 0)
      nilerror("malloc_debug", "Wasn't linked with real malloc_debug()");
}
