-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- File: stdio.d
-- Author: Andy Lowry
-- SCCS Info: @(#)stdio.d	1.2 3/13/90

-- Standard interfaces for input and output of characters and strings,
-- based on the interfaces incuded in the terminalIO definitions
-- module

stdio: using (terminalIO)
definitions
  
  stdin: record (
    getChar: getCharFunc,
    getString: getStringFunc
  );
  
  stdout: record (
    putChar: putCharFunc,
    putString: putStringFunc,
    putLine: putStringFunc
  );
  
end definitions
