-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- SCCS Info: @(#)findfile.d	1.3 3/13/90

findFile: using (unix) definitions

  findfile_Intf: callmessage (
    file: charstring,
    mode: unix!access,
    foundfile: charstring
  ) constant (file, mode)
    exit { full }
    exception file_Not_Found { init(file), init(mode) };

  findfile_Q: inport of findfile_Intf { init(file), init(mode) };
  findfile_Func: outport of findfile_Q;

end definitions
