/* (C) Copyright International Business Machines Corporation 23 January */
/* 1990.  All Rights Reserved. */
/*  */
/* See the file USERAGREEMENT distributed with this software for full */
/* terms and conditions of use. */
/* SCCS Info: @(#)asmfunc.h	1.2 3/13/90 */

void
init_asm();

void
init_proc();

void
init_stmt();

void
init_operand();

void
add_component();

void
add_operand();

void
q_boolean();

void
q_integer();

void
q_charstring();

char *
copystring();

void
free_string();
