# (C) Copyright International Business Machines Corporation 23 January 
# 1990.  All Rights Reserved. 
#  
# See the file USERAGREEMENT distributed with this software for full 
# terms and conditions of use. 
# File: fmtopcode.awk
# Author: David F. Bacon
# SCCS Info: @(#)fmtopcode.awk	1.3 3/13/90

# take interface/operator.table and generate the code to map enums to names

{
	print "where ('" $1 "') FP.FormatValue := \"" $1 "\";"
}
