-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
optimize: using (predefined,cginternal) definitions
    BBidBag: table of BBid {init} keys (*);
    BBidSubstitution: record (
	old: BBidBag,
	new: BBid
      );
    EmptyBBRemoval: table of BBidSubstitution {full} keys (new);
end definitions
