-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- SCCS Info: @(#)posmap.d	1.2 3/13/90

posmap: using (predefined, positions) definitions

  position_mappings: table of position_mapping { full } keys (id);

  position_mapping: record (
    id: executable_id,
    mapping: posmaps
  );

  posmaps: table of posmap { full } keys (aposition);

  posmap: record (
    aposition: aposition,		-- position in absprog (clause,stmt)
    cposition: cposition		-- position in source text (line,col)
  );

-- note something must be done about reporting errors in definitions modules 
-- as well, since part of that checking will be done in Hermes.

end definitions
