-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- SCCS Info: @(#)initfindfile.d	1.2 3/13/90

initFindFile: using (findFile, unix, cwd) definitions

  initFindFile: callmessage (
    access: unix!access_Func,
    getCwd: cwd!getCwdFn,
    path: charstring,
    pathfunc: findfile_Func
  ) constant (access, getCwd, path)
    exit { full };

  initFindFileQ: inport of initFindFile
    { init(access), init(getCwd), init(path) };
  initFindFileFn: outport of initFindFileQ;

end definitions
