/* (C) Copyright International Business Machines Corporation 23 January */
/* 1990.  All Rights Reserved. */
/*  */
/* See the file USERAGREEMENT distributed with this software for full */
/* terms and conditions of use. */
/* File: rpc.h */
/* Author: David F. Bacon */
/* SCCS Info: @(#)rpc.h	1.3 3/13/90 */

#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <rpc/rpc.h>
#include <sys/time.h>

#include "cfunc.h"

#include "distributed.cd"

xdr_status hxdr_object();

#define is_bottom(obj) ((obj)->tsdr is &dr_bottom)

