/* (C) Copyright International Business Machines Corporation 23 January */
/* 1990.  All Rights Reserved. */
/*  */
/* See the file USERAGREEMENT distributed with this software for full */
/* terms and conditions of use. */
/* SCCS Info: @(#)accessors.h	1.3 3/13/90 */

#define dot(valc, off) (valc.record->data[off].value)
#define dcdot(valc, off) (valc->data[off].value)
#define drdot(valc, off) (valc.record->data[off].tsdr)

#define vec_elem(tbl, idx) \
  ((tbl)->tbls[0].rep.vec->elements[idx])
#define get_elem(tbl, idx) (vec_elem((tbl).table,idx))

#define case_of(var) (var.variant->info.variant_case)
#define size_of(valc) ((valc).table->size)
