-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- data local to the resource manager

rmInternal: using (RManager, common) 
definitions
    
  resourceRepository: table of resourceDescriptor {full} keys (name);
    
  resourceDescriptor: record (
    owner: charString,		-- name of user who posted resource
    name: charString,		-- user-given name of resource
    resource: polymorph,	-- resource given by user
    access: accessFn		-- access control function from user
  );
    
end definitions



