-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- File: bbassemble.d
-- Author: Andy Lowry
-- SCCS Info: @(#)bbassemble.d	1.2 3/13/90

-- Type definitions needed by the BBAssemble module

BBAssemble: using (cgInternal)

definitions
  
  -- A BBAddr record associates a starting address with a
  -- particular BB.  The starting address is an offset relative to the
  -- beginning of the assembled LI code.
  BBAddr: record (
    BBid: BBid,
    addr: integer
  );
  BBAddrs: table of BBAddr {full} keys (BBid);
  
end definitions
