/* (C) Copyright International Business Machines Corporation 23 January */
/* 1990.  All Rights Reserved. */
/*  */
/* See the file USERAGREEMENT distributed with this software for full */
/* terms and conditions of use. */
/* SCCS Info: @(#)stack.h	1.3 3/13/90 */

void stack_create();
void stack_push();
object * stack_top();
object * stack_pop();
void stack_popndrop();
object * stack_popntop();
object * stack_next();


typedef struct stack_struct
{
    struct stack_struct *next;
    object *value;
} stack, *stackp, **stackpp;
