-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- File: itoa.d
-- Author: Andy Lowry
-- SCCS Info: @(#)itoa.d	1.2 3/13/90

-- Routine to convert an integer into a string of digits, needed by
-- formatError

itoa: using ()
definitions

  itoa: callmessage (
    int: integer,
    str: charString
  )
  constant (int)
  exit {full};
  itoaQ: inport of itoa {init(int)};
  itoaFn: outport of itoaQ;

end definitions
  
