-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- File: profile.d
-- Author: David F. Bacon
-- SCCS Info: @(#)profile.d	1.2 3/13/90

profile: using () definitions
    
    profiles: table of profile { full } keys (id);

    profile: record (
	id: processid,		-- uniqueid of process
	name: charstring,	-- name of process
	creates: integer,	-- how many times created
	counts: integerList	-- execution count per Li operation
      );

    integerList: ordered table of integer { init };
    
  end definitions

