/* (C) Copyright International Business Machines Corporation 23 January */
/* 1990.  All Rights Reserved. */
/*  */
/* See the file USERAGREEMENT distributed with this software for full */
/* terms and conditions of use. */
/* File: resolve.h */
/* Author: David F. Bacon */
/* SCCS Info: @(#)resolve.h	1.3 3/13/90 */

#include "resdefs.h"

extern objectp
  Absprog,
  Typedefinitions,
  Attrdefinitions,
  Currentdefinition;

void fe_error();		/* used instead of yyerror() */

char *tempalloc();		/* for temporary storage alloc/deallocation */
void tempfree();

#define tnew(t) ((t *) tempalloc(sizeof(t)))

char *copystring();
char *downcase();

objectp new_object();
