-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
copy: using (predefined,common,terminalio)
  definitions
    copy: callmessage (
	in: getStringFunc,
	out: putStringFunc
      )
    constant (in,out)
    exit { init(in),init(out)};
    copyQ: inport of copy { init(in),init(out) };
    copyFn: outport of copyQ;
  end definitions

