# (C) Copyright International Business Machines Corporation 23 January \
# 1990.  All Rights Reserved. \
#  \
# See the file USERAGREEMENT distributed with this software for full \
# terms and conditions of use. \
# SCCS Info: @(#)getsys.sed	1.3 3/14/90

1,/syscall: enumeration/d
/);/,$d
