# (C) Copyright International Business Machines Corporation 23 January 
# 1990.  All Rights Reserved. 
#  
# See the file USERAGREEMENT distributed with this software for full 
# terms and conditions of use. 
BEGIN {
	FS = "|"
	opcode = 0
}


/^[a-z]/ {
	if (index("ABCDEFGHIJKLMNOPQRSTUVWXYZ", substr($0, 1, 1)) != 0)
	  next
	endop = index($1, " ")-1
	if (endop < 1)
	  endop = length($1)

	op = substr($1, 1, endop)
	print "      where (interpform!opcode#'" op "')" > "opcodes.h"
	print "        s <- charstring#\"" op "\";" > "opcodes.h"
	opcode++
}
