-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
chainedRM: using (rManager, sysRManager) definitions
    
  rmList: ordered table of rManager { full };
  sysRMList: ordered table of sysRManager {full};
    
  chainedRMInit: callmessage (
    rmList: rmList,		-- list of rm's to chain together
    rm: rManager		-- the single RM that fronts for all of them
  )
  constant (rmList)
  exit { full };
    
  chainedRMInitQ: inport of chainedRMInit { init(rmList) };
  chainedRMInitFn: outport of chainedRMInitQ;
    
  chainedSysRMInit: callmessage (
    rmList: sysRMList,
    rm: sysRManager
  )
  constant (rmList)
  exit {full};
  
  chainedSysRMInitQ: inport of chainedSysRMInit {init(rmList)};
  chainedSysRMInitFn: outport of chainedSysRMInitQ;

end definitions

