-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- File: filedef.d
-- Author: David F. Bacon
-- SCCS Info: @(#)filedef.d	1.4 3/13/90

filedef: using (parse, posmap)
definitions

  filed_definition: record (
    definitions_module: predefined!definitions_module,
    defmap: predefined!definitions_printmap,
    procmaps: predefined!executable_printmappings,
    direct_imports: module_printmap );



  filed_process: record (
      main_program: processid,
      programs: processes,
      procmaps: predefined!executable_printmappings,
      direct_imports: module_printmap,
      links: linkedPrograms,
      posmaps: position_mappings
  );

end definitions
