-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- File: personalrm.d
-- Author: Andy Lowry
-- SCCS Info: @(#)personalrm.d	1.2 3/13/90

-- Interface for a process that spawns a new empty resource manager,
-- filtered so as to make all requests tagged with the given user
-- name. 

personalRM: using (rManager)

definitions
  
  personalRM: callmessage (
    userName: charString,
    rm: rManager		-- capas to the personal rm
  )
  constant (userName)
  exit {full};
  
  personalRMQ: inport of personalRM {init(userName)};
  personalRMFn: outport of personalRMQ;
  
end definitions
