-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- SCCS Info: @(#)errprint.d	1.2 3/13/90

errprint: using (predefined, errors) definitions 

  err_printtable: table of errprint { full } keys (code);

  errprint: record (
    code: errorcode,
    printtable: printtable );

  printtable: ordered table of printblock { full };

  printblock: record (
    string: charstring,			-- string to print
    optional_obj: opt_objposition );	-- object to print or nothing

  opt_objposition: variant of option (
    'absent' -> empty: empty {},	-- print no object, just the string
    'present' -> position: integer { init }
					-- position (in table) of ErrorObject 
  );					--   to print.
    

end definitions
