-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- File: compoffsets.d
-- Author: Andy Lowry
-- SCCS Info: @(#)compoffsets.d	1.2 3/13/90

-- Type definitions specific to the compOffsets process


compOffsets: using ()

definitions

  -- A componentMap relates a particular component of a given type to
  -- the type of the component and the offset in the parent type's data
  -- representation where the component object is stored.
  componentMap: table of componentMapEntry {full} keys (type,comp);
  componentMapEntry: record (
    type: typename,
    comp: componentid,
    offset: integer,
    compType: typename
  );
  
end definitions

