-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- SCCS Info: @(#)annotate.d	1.2 3/13/90

annotate: using (predefined, common) definitions

  annotations: table of annotation { full } keys (name);

  annotation: record (
    name: charstring,
    thing: polymorph
  );

end definitions
