-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
plumber: using (plumbing, rmanager) definitions
    
    plumber: callmessage (
	rm: rManager,		-- resource manager to use for pathLoad, etc
	procs: procSet,		-- procs to create
	pipes: pipeSet,		-- pipes to create
	procRM: rManager,	-- resource manager to give to each proc
	asynchronous: boolean	-- return right away or wait till all procs
				--  finish and return.
      )
    constant (procs, pipes, rm, procRM, asynchronous)
    exit { full };
    
    plumberQ: inport of plumber { full };
    plumberFn: outport of plumberQ;

  end definitions

