-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- SCCS Info: @(#)positions.d	1.3 3/13/90

positions: USING (predefined) 
  DEFINITIONS

  aposition: record (		-- a position in an abstract program
    clause: clauseid,
    statement: statementid );

  cposition: record (		-- a position in the text of a program
    file: charstring,
    line: integer,
    column: integer
  );

  position: variant of positionType (-- one of the above
    'absprog' -> apos: aposition {full},
    'source' -> cpos: cposition {full}
  );
  positionType: enumeration ('absprog', 'source');

END DEFINITIONS
