/* (C) Copyright International Business Machines Corporation 23 January */
/* 1990.  All Rights Reserved. */
/*  */
/* See the file USERAGREEMENT distributed with this software for full */
/* terms and conditions of use. */
/* File: parse.h */
/* Author: David F. Bacon */
/* SCCS Info: @(#)parse.h	1.2 3/13/90 */


/* subclass of cprocessdata in include/li.h: */

typedef struct {
    object initport;		/* the initialization port */

    object serviceport;		/* parseDefQ or parseProcQ */

    object portrec;		/* record of ports to services used by */
				/*  the front end. */
} feprocessdata;
