-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
disopcode: using (predefined,interpform,disassembler,disinternal)
  process (Q: disopcodeQ)
  declare
    args: disopcode;
    s: charstring;
  begin
    receive args from Q;
    select (args.opcode)
#include "opcodes.h"
      otherwise
	print charstring#"Unknown opcode.";
	exit Unsupported;
      end select;
    -- print charstring#"Printing operation:";
    -- print s;
    args.rep <- s;
    return args;
  on exit (Unsupported)
    return args exception Unsupported;
  end process
