/* (C) Copyright International Business Machines Corporation 23 January */
/* 1990.  All Rights Reserved. */
/*  */
/* See the file USERAGREEMENT distributed with this software for full */
/* terms and conditions of use. */
/* SCCS Info: @(#)asm.h	1.2 3/13/90 */

typedef struct env_struct {
    struct env_struct *next;

    objectp prog;
    objectp stmt;
    objectp labels;
    objectp backpatch;
    objectp exits;
} env;
