/* (C) Copyright International Business Machines Corporation 23 January */
/* 1990.  All Rights Reserved. */
/*  */
/* See the file USERAGREEMENT distributed with this software for full */
/* terms and conditions of use. */
#define _BSD 43

#include <rpc/rpc.h>

enum clnt_stat clnt_send( /* h, proc, xdr_args, args_ptr */ );

CLIENT *clntatcp_create( /* raddr, prog, vers, sockp, sendsz, recvsz */ );
CLIENT *clntatcp_cachedcreate( /* raddr, prog, vers, sockp, sendsz,recvsz */ );

SVCXPRT *svcatcp_create( /* sock, sendsize, recvsize */ );
