# (C) Copyright International Business Machines Corporation 23 January 
# 1990.  All Rights Reserved. 
#  
# See the file USERAGREEMENT distributed with this software for full 
# terms and conditions of use. 
s/^include .*includefile/# include includefile\
MAKE=make -f Mkfile\
/
/# include includefile/r make/includefile
