-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- File: stringpipe.d
-- Author: Andy Lowry
-- SCCS Info: @(#)stringpipe.d	1.2 3/13/90

-- The ends of a pipe for transmitting charstrings

stringpipe: using (common)

definitions
  
  stringQ: inport of charstring {init};
  stringPort: outport of stringQ;

  stringPorts: table of stringPort {init};

end definitions
