/* (C) Copyright International Business Machines Corporation 23 January */
/* 1990.  All Rights Reserved. */
/*  */
/* See the file USERAGREEMENT distributed with this software for full */
/* terms and conditions of use. */
/* SCCS Info: @(#)lex-yacc.h	1.2 3/13/90 */

/* interface between lex and yacc parts of front end */

typedef struct
{
    int lineno;
    int charno;
    int tokno;
} globinfo;

