-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- SCCS Info: @(#)pathload.d	1.3 3/13/90

pathload: using (predefined, load, findfile) definitions

  pathload_init_intf: callmessage (
    loader: load_func,
    pathfinder: findfile_func,
    pathloader: load_func
  ) constant (loader, pathfinder)
    exit { full };

  pathload_init_q: inport of pathload_init_intf 
    { init(loader), init(pathfinder) };
  pathload_init_func: outport of pathload_init_q;

-- pathloader and load both have the same interface -- see load.d

end definitions
