/* (C) Copyright International Business Machines Corporation 23 January */
/* 1990.  All Rights Reserved. */
/*  */
/* See the file USERAGREEMENT distributed with this software for full */
/* terms and conditions of use. */
/* File: trigger.h */
/* Author: David F. Bacon */
/* SCCS Info: @(#)trigger.h	1.2 3/13/90 */

/* this structure is used when the interpreter is called recursively from */
/* within a C-process.  the "trigger" is the caller, and when it is resumed, */
/* we return out of the recursion. */

typedef struct trigger_struct {
    struct trigger_struct *next;
				/* subclass of list */
    pcb *trigger;		/* the process to trigger on */
    JumpBuf jbuffer;		/* where to go when we pull the trigger */
    object *Exception;		/* the exception raised on return */
} trig;

