-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- SCCS Info: @(#)initcwd.d	1.2 3/13/90

initCwd: using (cwd)

definitions

  initCwd: callmessage (
    directory: charstring,
    setCwd: setCwdFn,
    getCwd: getCwdFn
  ) constant (directory)
    exit { full };

  initCwdQ: inport of initCwd { init(directory) };
  initCwdFn: outport of initCwdQ;

end definitions
