/* (C) Copyright International Business Machines Corporation 23 January */
/* 1990.  All Rights Reserved. */
/*  */
/* See the file USERAGREEMENT distributed with this software for full */
/* terms and conditions of use. */
#ifndef lint
static char sccsinfo[] = "@(#)o_bottom.c	1.6 3/13/90";
#endif

#include "li.h"

/*ARGSUSED*/
void
fin_bottom(v, f_op, sched)
valcell v;
finalize_op f_op;
schedblock *sched;
{
}


/*ARGSUSED*/
predef_exception
cp_bottom(dst, src)
valcell *dst, src;
{
    dst->integer = 0;
    return(Normal);
}


/*ARGSUSED*/
status
eq_bottom(v1,v2)
valcell v1, v2;
{
    return(SUCCESS);		/* two uninit things are always equal */
}
