/* (C) Copyright International Business Machines Corporation 23 January */
/* 1990.  All Rights Reserved. */
/*  */
/* See the file USERAGREEMENT distributed with this software for full */
/* terms and conditions of use. */

/* These processes are server like */

typedef struct fdreaddata_struct {
  object initport;
  object initmessage;
  object callport;
  object callmessage;
  int fd;
} FDREADDATA;

typedef struct fdwritedata_struct {
  object initport;
  object initmessage;
  object callport;
  object callmessage;
  int fd;
  char *buffer;
  int count;
} FDWRITEDATA;

typedef struct fdclosedata_struct {
  object initport;
  object initmessage;
  object callport;
  object callmessage;
  int fd[2];
} FDCLOSEDATA;
