-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- File: fmtopcode.pp
-- Author: Robert E. Strom
-- SCCS Info: @(#)fmtopcode.pp	1.5 3/13/90

FmtOpcode : USING(AbsFormat)
PROCESS (FormatOpcodeInit : FormatOpcodeInport )
  DECLARE
    FP: FormatOpcodeCall ;
  BEGIN
    RECEIVE FP FROM FormatOpcodeInit;
    SELECT (FP.Operator)
#include "fmtopcode.body"
      OTHERWISE
        FP.FormatValue <- "UNKNOWN";
      END SELECT;
    RETURN FP;
  END PROCESS
