-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
launcher: using (rmain, rManager, common) definitions
    
    launcher: callmessage (
	proc: rmainFn,
	rm: rManager,
	signaler: signalPort,
	asynchronous: boolean
      )
    constant (proc, rm, signaler, asynchronous)
    exit { full };
    
    launcherQ: inport of launcher { full };
    launcherFn: outport of launcherQ;
    
  end definitions

