/* (C) Copyright International Business Machines Corporation 23 January */
/* 1990.  All Rights Reserved. */
/*  */
/* See the file USERAGREEMENT distributed with this software for full */
/* terms and conditions of use. */
#include <suntool/sunview.h>

main()
{
  Frame frame;
  frame = window_create(NULL,FRAME,FRAME_LABEL,"hello world.",0);
  window_main_loop(frame);
}
