-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- File: progfixfull.d
-- Author: David F. Bacon
-- SCCS Info: @(#)progfixfull.d	1.3 3/13/90

progFixFull: using () definitions
    
    progFixFull: callmessage (
	prog: program
      )
    exit { full };
    
    progFixFullQ: inport of progFixFull { full };
    progFixFullFn: outport of progFixFullQ;
    
  end definitions

