-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- SCCS Info: @(#)resproc.d	1.3 3/13/90

resproc: using (predefined) definitions

  expression_clause_info: record (
      result: objectname,
      clause: clauseid
    );


  handler_set: table of handlername { full } keys (*);

  
  builtin_exception_mapping: table of builtin_exception_map { full }
    keys (name);

  builtin_exception_map: record (
      name: charstring,
      exception: builtin_exception
    );

end definitions
