-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 

safewindowgen: using (terminalio, common, safewindow)
  definitions
    
    safewindowgen: callmessage (
	terminal: terminalFunctions, -- access to window,
	close: signalPort, -- inform window that application is dying
	safewindowinport: terminalQ -- port to send terminalCM
      )
    constant (terminal, close, safewindowinport)
    exit {full};
    safewindowgenQ: inport of safewindowgen 
      {full};
    safewindowgenFn: outport of safewindowgenQ;
    
    
  end definitions
