-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- SCCS Info: @(#)inferredtype.d	1.2 3/13/90

inferredtype:  USING (predefined)
DEFINITIONS
  InferredDefinitions: table of InferredDefinition { full } keys (root);

  InferredDefinition: record (
    root: rootname,		-- (scopeid, rootid) of object
    type: typename );		-- its inferred type

END DEFINITIONS
