-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- SCCS Info: @(#)common.d	1.2 3/13/90

common: using (predefined) definitions

  polymorph: polymorph;


  charstringList: ordered table of charstring { init };


  signalQ: inport of empty { };

  signalPort: outport of signalQ;

end definitions
