-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
cwindow: using (predefined,unix)
  definitions

    -- definitions for the chermes processes
    cWindow: callmessage (
	label: charstring,
	options: charstring,
	in: unix!handle,
	out: unix!handle
      )
    constant (label)
    exit {full};
    cWindowQ: inport of cWindow {init(label)};
    cWindowFn: outport of cWindowQ;

  end definitions
