-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 

safewindowinit: using (main, CLoad, safewindow)
  definitions
    
    safewindowinit: callmessage (
        CLoader: CLoad!CLoadFn, -- C Loader
        open: safewindowFn -- port to open-window service
      )
    constant (CLoader)
    exit {full};
    safewindowinitQ: inport of safewindowinit 
      {init(CLoader)};
    safewindowinitFn: outport of safewindowinitQ;
    
  end definitions
