-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 

window: using (predefined, common, cload, terminalio)
  definitions
    
    window: callmessage (
	CLoader: CLoad!CLoadFn,
	label: charstring,
	options: charstring,
	terminal: terminalFunctions,
	close: signalPort
      )
    constant (label,options,CLoader)
    exit {full}
    exception Failure {init(label),init(options),init(CLoader)};
    
    windowQ: inport of window 
      {init(label),init(options),init(CLoader)};
    windowFn: outport of windowQ;
    
  end definitions
