-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- SCCS Info: @(#)getfile.d	1.2 3/13/90

getFile: using (unix, predefined) definitions

  getFile: callmessage (
    name: charstring,
    file: charstring
  ) constant (name)
    exit { full }
    exception cantRead { init(name) };

  getFileQ: inport of getFile { init(name) };
  getFileFunc: outport of getFileQ;


  getFileInit: callmessage (
    fopen: fopen_function,
    fclose: fclose_function,
    fread: fread_function,
    getFile: getFileFunc
  ) constant (fopen, fclose, fread)
    exit { full };

  getFileInitQ: inport of getFileInit
    { init(fopen), init(fclose), init(fread) };
  getFileInitFunc: outport of getFileInitQ;

end definitions
