-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- File: worklist.d
-- Author: Daniel Yellin
-- SCCS Info: "@(#)worklist.d	1.2 3/13/90"
-- description:  definitions for worklist used by make.p

worklist: using(make)
definitions
 
worklist: table of node {init(name), init(successors)} keys(name);
-- node is defined in make.d

nodeinfoset: table of nodeinfo { init(name), init(unmadepred) } keys(name);

nodeinfo: record(
  name: charstring,
  unmadepred: integer);

end definitions

