-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- SCCS Info: @(#)cwd.d	1.2 3/13/90

-- Definitions for the process that represents the "current working directory"
--  on antiquated systems like Unix, DOS, Tops-20, etc.

cwd: using (predefined) definitions

  getCwd: callmessage (
    cwd: charstring
  ) exit { init(cwd) };

  getCwdQ: inport of getCwd {};
  getCwdFn: outport of getCwdQ;


  setCwd: callmessage (
    directory: charstring
  ) constant (directory)
    exit { init(directory) }
    exception badDirectory { init(directory) };

  setCwdQ: inport of setCwd { init(directory) };
  setCwdFn: outport of setCwdQ;

end definitions
