-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- File: checkdefs.d
-- Author: David F. Bacon
-- SCCS Info: @(#)checkdefs.d	1.2 3/13/90

checkDefs: using (errors) definitions
    
    checkDefs: callmessage (
	id: moduleid,
	defs: definitions_modules,
	errors: errors
      ) constant (id, defs)
    exit { full };
    
    checkDefsQ: inport of checkDefs { init(id), init(defs) };
    checkDefsFn: outport of checkDefsQ;
    
  end definitions

