-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- File: fixfull.d
-- Author: Rob Strom
-- SCCS Info: @(#)fixfull.d	1.2 3/13/90

-- interface to Fixfull and Shorten
FixFull: USING() 
definitions
-- procedure to compute an adds/drops given a pre-condition/post-condition pair
FixFullCall: CALLMESSAGE (
  Definitions: Definitions_Modules, -- modules for resolution
  Formal: Formal_Typestate, -- a formal typestate which is to be expanded(fixfull) or contracted(shorten)
  RootType: TypeName -- the typename of implied root of formal
  
)
CONSTANT(Definitions, RootType) 
EXIT{full}
MINIMUM{full};

FixFullInport: INPORT OF FixFullCall {Full};
FixFullOutport: OUTPORT OF FixFullInport;

end definitions
