-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- File: cgstmt.d
-- Author: Andy Lowry
-- SCCS Info: @(#)cgstmt.d	1.2 3/13/90

-- Internal definitions for cgStmt

cgStmt: using (cgInternal)

definitions

  -- the adhocStmtMap maps statement operator codes to capabilities to
  -- translate those statements
  adhocStmtMap: table of adhocStmtMapEntry {full} keys (opcode);
  adhocStmtMapEntry: record (
    opcode: predefined!operator,
    translator: cgStmtFn
  );
  
end definitions

