# (C) Copyright International Business Machines Corporation 23 January 
# 1990.  All Rights Reserved. 
#  
# See the file USERAGREEMENT distributed with this software for full 
# terms and conditions of use. 
# SCCS Info: @(#)oplex.awk	1.3 3/14/90

BEGIN {
  print "/* Automatically generated from trans/greencard by oplex.awk */\n\n"
  x = 0
}

/^[a-z]/ {
  if (index("ABCDEFGHIJKLMNOPQRSTUVWXYZ", substr($0, 1, 1)) != 0)
    next
  print $1 "        { yyop = " x "; Treturn(OPCODE); }"
  x = x + 1
}
