-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- File: objectcache.d
-- Author: David F. Bacon
-- SCCS Info: @(#)objectcache.d	1.3 3/13/90

objectCache: using (objectIO, unixStat) definitions

-- creating an object cache: the normal readObject and writeObject functions
-- are replaced by the acahing versions.

  initCache: callmessage (
    read: readObject_Func,
    write: writeObject_Func,
    stat: statMtimeFunc
  ) exit { full };

  initCacheQ: inport of initCache { full };
  initCacheFunc: outport of initCacheQ;

end definitions
