-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- File: fixfullint.d
-- Author: Rob Strom
-- SCCS Info: @(#)fixfullint.d	1.2 3/13/90

-- internal information used by Fixfull
FixFullInt: USING() 
definitions
-- given a definition, and a formal object, put init of it and its
-- descendendts into formal typestate
ExpandFullCall: CALLMESSAGE (
  Definitions: Definitions_Modules,
  Formal: Formal_Typestate, 
  FormalObject: Component_list,
  Type: TypeName,
  Self: ExpandFullOutport -- way to call self recursively
)
CONSTANT(Definitions, FormalObject, Type, Self) 
EXIT{full};
ExpandFullInport: INPORT OF ExpandFullCall {Full};
ExpandFullOutport: OUTPORT OF ExpandFullInport;

end definitions
