-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- File: booleanpipe.d
-- Author: Andy Lowry
-- SCCS Info: @(#)booleanpipe.d	1.2 3/13/90

-- The ends of a pipe for transmitting booleans

booleanPipe: using ()

definitions

  booleanQ: inport of boolean {init};
  booleanPort: outport of booleanQ;

  booleanPorts: table of booleanPort {init};

end definitions
