# (C) Copyright International Business Machines Corporation 23 January 
# 1990.  All Rights Reserved. 
#  
# See the file USERAGREEMENT distributed with this software for full 
# terms and conditions of use. 
# SCCS Info: @(#)tokens.awk	1.2 3/13/90
#
# Expects the concatenation of the files "keywords" and "othertokens" as input,
# generates:
#
# tokens.yacc
#    which has a "%token" definition which assigns a number to each token.
#
/^[A-Z_]+/ {
  print "%token " $1
}
