# (C) Copyright International Business Machines Corporation 23 January 
# 1990.  All Rights Reserved. 
#  
# See the file USERAGREEMENT distributed with this software for full 
# terms and conditions of use. 
# SCCS Info: @(#)opcodes.awk	1.4 3/14/90

BEGIN {
  print "-- Automatically generated from trans/greencard by opcodes.awk\n\n"
  x = 0
}

/^[a-z]/ {
  if (index("ABCDEFGHIJKLMNOPQRSTUVWXYZ", substr($0, 1, 1)) != 0)
    next
  if (x==1)
    print "    ,"

  endop = index($1, "|")-1
  if (endop < 1)
    endop = length($1)
  op = substr($1, 1, endop)

  x=1
  print "    '" op "'"
}
