-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- File: cgstmtstub.pp
-- Author: Andy Lowry
-- SCCS Info: @(#)cgstmtstub.pp	1.5 3/13/90

-- This procedure is a stub that acts like a statement translator, but
-- does nothing to the basic block structure being built.  It takes
-- the place of not-yet-implemented statement translators.

#include "typemark.h"
#include "codegen.h"

cgStmtStub: using (cgInternal)

process (Q: cgStmtQ)

declare
  args: cgStmt;
begin
  receive args from Q;		-- get the request
  return args;			-- and bounce it back without doing anything
end process
