-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- SCCS Info: @(#)pathreadobj.d	1.3 3/13/90

pathreadobj: using (predefined, objectio, findfile) definitions

  pathreadobj_init_intf: callmessage (
    readobj: readobject_func,
    pathfinder: findfile_func,
    pathreadobj: readobject_func
  ) constant (readobj, pathfinder)
    exit { full };

  pathreadobj_init_q: inport of pathreadobj_init_intf 
    { init(readobj), init(pathfinder) };
  pathreadobj_init_func: outport of pathreadobj_init_q;

-- pathreadobj and readobj both have the same interface -- see objectio.d

end definitions
