/* (C) Copyright International Business Machines Corporation 23 January */
/* 1990.  All Rights Reserved. */
/*  */
/* See the file USERAGREEMENT distributed with this software for full */
/* terms and conditions of use. */
/* SCCS Info: @(#)tablefuncs.h	1.4 3/13/90 */

#define va_tableargs(argv, thetable, tblnum, func, elcnt, expected) \
  va_list argv; dfd_table * thetable; trepnum tblnum; int (* func)(); \
  counter *elemcounter; int expected; \
  va_start(argv); \
  thetable = va_arg(argv, dfd_table *); tblnum = va_arg(argv, counter); \
  func = (int (*)()) va_arg(argv, int *); \
  elemcounter = va_arg(argv, counter *); \
  expected = va_arg(argv, int);



