-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- Tables for associating arbitrary comment text with any named type
-- or constructor.  The comments should appear with any Hermes type
-- definitions produced from the C type specifications
-- Andy Lowry & David F. Bacon, Apr 1989

-- SCCS Info: @(#)typecomments.d	1.2 3/13/90

typeComments: using (predefined)
definitions

  typeComments: table of typeComment {full} keys (name);

  typeComment: record (
    name: charstring,
    comment: charstring
  );

end definitions
