-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- File: uptolow.d
-- Author: Daniel M. Yellin
-- SCCS info: "@(#)uptolow.d	1.2 3/13/90"
uptolow: using()
definitions
 
uptolowIntf: callmessage(
   string: charstring
)  exit {full};
uptolowQ: inport of uptolowIntf {init(string)};
uptolowFn: outport of uptolowQ;

end definitions
