/* (C) Copyright International Business Machines Corporation 23 January */
/* 1990.  All Rights Reserved. */
/*  */
/* See the file USERAGREEMENT distributed with this software for full */
/* terms and conditions of use. */
/* SCCS Info: @(#)main.c	1.2 3/13/90 */

#include <stdio.h>

main()
{
    yyparse();
}


yyerror(s)
char *s;
{
    fprintf(stderr, "%s\n", s);
}
