# (C) Copyright International Business Machines Corporation 23 January 
# 1990.  All Rights Reserved. 
#  
# See the file USERAGREEMENT distributed with this software for full 
# terms and conditions of use. 
# SCCS Info: @(#)reswords.awk	1.2 3/13/90
#
# takes all the words in thew file "reswords" and concatenates them, surrounded
# by parentheses and separated by "|".  the result is used as an argument to
# grep with the -v (avoid) option to remove reserved words from the "keywords"
# file.
#
BEGIN { out = "" }
/^[A-Z]+/ {
  out = out "(" $1 ")|"
}
END { print "^(" substr(out, 1, length(out)-1) ")$" }
