/* (C) Copyright International Business Machines Corporation 23 January */
/* 1990.  All Rights Reserved. */
/*  */
/* See the file USERAGREEMENT distributed with this software for full */
/* terms and conditions of use. */
/* File: o_undone.c */
/* Author: David F. Bacon */
#ifndef lint
static char sccsinfo[] = "@(#)o_undone.c	1.4 3/13/90";
#endif


/* this file contains stubs for unimplemented back-end operations.  currently,
   constraints, checkdefinitions, and reals are completely unsupported.  also,
   no type/typestate information is kept about polymorphs, so that they are
   completely unchecked and the TYPE OF and TYPESTATE OF operations are 
   unimplemented.
 */

 void o_assert(){}
 void o_drop(){}

 void o_checkdefinitions(){}

 void o_type_of(){}
 void o_typestate_of(){}

 void o_oecvt_real(){}
 void o_icvt_real(){}
 void o_rlit(){}
 void o_radd(){}
 void o_rsubtract(){}
 void o_rmultiply(){}
 void o_rdivide(){}
 void o_rgt(){}
 void o_rlt(){}
 void o_rge(){}
 void o_rle(){}
 void o_rcvt_int(){}
 void o_rcvt_enum(){}
 void o_rnegate(){}
