-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- SCCS Info: @(#)printobj.d	1.2 3/13/90

printobj: using (common) definitions 

  printIntf: callmessage (
    object: polymorph
  ) constant (object)
    exit { init(object) };

  printQ: inport of printIntf { init(object) };
  printFunc: outport of printQ;

end definitions

