-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- File: optimize.pp
-- Author: Andy Lowry
-- SCCS Info: @(#)optimize.pp	1.2 3/13/90

-- This process performs various optimizations on the basic-block
-- representation of a compiled Hermes program

-- *** CURRENTLY NOTHING IS DONE ***

#include "typemark.h"
#include "codegen.h"

optimize: using (cgInternal)

process (Q: optimizeQ)
  
declare
  args: optimize;
begin
  receive args from Q;
  return args;
end process
    
