-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- SCCS Info: @(#)objectio.d	1.2 3/13/90

objectIO: using (predefined, common)
definitions

  readobject_intf: callmessage (
    filename: charstring,
    object: polymorph
  ) constant (filename)
    exit { full }
    exception file_not_found { init(filename) }
    exception file_not_readable { init(filename) };

  readobject_q: inport of readobject_intf { init(filename) };
  readobject_func: outport of readobject_q;


  writeobject_intf: callmessage (
    filename: charstring,
    object: polymorph
  ) constant (filename, object)
    exit { full }
    exception object_not_writeable { full }
    exception file_not_writeable { full };

  writeobject_q: inport of writeobject_intf { full };
  writeobject_func: outport of writeobject_q;

end definitions
