-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 

safewindow: using (terminalio)
  definitions
    
    safewindow: callmessage (
	label: charstring, -- stripe label
	options: charstring, -- other options
	terminal: terminalCM -- access to window,
      )
    constant (label,options)
    exit {full}
    exception Failure {init(label),init(options)};
    safewindowQ: inport of safewindow 
      {init(label),init(options)};
    safewindowFn: outport of safewindowQ;
    
    terminalCM: callmessage (
        terminal: terminalFunctions
      )
    constant (terminal) exit {full};
    terminalQ: inport of terminalCM {full};
    terminalFn: outport of terminalQ;
    
  end definitions
