# (C) Copyright International Business Machines Corporation 23 January 
# 1990.  All Rights Reserved. 
#  
# See the file USERAGREEMENT distributed with this software for full 
# terms and conditions of use. 
# SCCS Info: @(#)opt.awk	1.2 3/13/90

/^[a-z_]*$/ {
#  print "opt_" $1 " : " $1 " { $$ = $1; } ;"
#  print "opt_" $1 " : empty  { $$ = NULL_NODE; } ;"
  print "opt_" $1 " : " $1 " ;"
  print "opt_" $1 " : empty ;"
  print ""
}
