-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- SCCS Info: @(#)errdisplay.d	1.3 3/13/90

errDisplay: using (positions) definitions

  errorMessages: table of errorMessage { full } ;

  errorMessage: record (
    position: cposition,	-- position in source text
    message: charstring		-- diagnostic message
  );


  errDisplay: callmessage (
    messages: errorMessages
  ) constant (messages)
    exit { full } ;

  errDisplayQ: inport of errDisplay { full };
  errDisplayFn: outport of errDisplayQ;

end definitions
