/* (C) Copyright International Business Machines Corporation 23 January */
/* 1990.  All Rights Reserved. */
/*  */
/* See the file USERAGREEMENT distributed with this software for full */
/* terms and conditions of use. */
/* Reserved words for minic parser
** Andy Lowry, Apr 1989

** SCCS Info: @(#)reswords.h	1.2 3/13/90
**/

/* Each entry includes the text of the reserved word and its */
/* associated token symbol, in struct initializer format */

{ "void", T_VOID },
{ "int", T_INT },
{ "char", T_CHAR },
{ "float", T_FLOAT },
{ "double", T_DOUBLE },
{ "struct", T_STRUCT },
{ "union", T_UNION },
{ "enum", T_ENUM },
{ "long", T_LONG },
{ "short", T_SHORT },
{ "unsigned", T_UNSIGNED },
{ "auto", T_AUTO },
{ "extern", T_EXTERN },
{ "register", T_REGISTER },
{ "typedef", T_TYPEDEF },
{ "static", T_STATIC },
{ "goto", T_GOTO },
{ "return", T_RETURN },
{ "sizeof", T_SIZEOF },
{ "break", T_BREAK },
{ "continue", T_CONTINUE },
{ "if", T_IF },
{ "else", T_ELSE },
{ "for", T_FOR },
{ "do", T_DO },
{ "while", T_WHILE },
{ "switch", T_SWITCH },
{ "case", T_CASE },
{ "default", T_DEFAULT },
{ "entry", T_ENTRY },
