-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- SCCS Info: @(#)main.d	1.3 3/13/90

main: using (CLoad, common, root, unix, stdenv) definitions

-- interface to Hermes main programs (as called by root.p)

  main_Intf: callmessage (
    argv: common!charstringList,
    environ: root!environ,
    std: stdenv!stdenv,
    unix: unix!unix,
    CLoader: CLoad!CLoadFn
  ) constant (argv, environ, std, unix, CLoader)
    exit { full };

  main_Q: inport of main_Intf { full };
  main_Func: outport of main_Q;

end definitions
