-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- File: fixmin.d
-- Author: Rob Strom
-- SCCS Info: @(#)fixmin.d	1.2 3/13/90

-- definitions local to fixdefs.p
FixMin: USING(Fixfull) 
definitions
-- Procedure to put init(comp)
-- into Minimum TS provided it's in exit TS
-- and to do the same with init record components
FixMinCall: CALLMESSAGE (
  Component: componentid, -- component being checked
  Parent: Component_list, -- parent component which has been put into minimum
  Declarations: Component_Declarations, -- declarations where component should be found
  Definitions: Definitions_modules, -- definitions visible
  ExitTS: Formal_Typestate, -- exitTS
  MinimumTS: Formal_Typestate -- minimumTS  
)
CONSTANT(Component, Parent, Declarations, Definitions, ExitTS) 
EXIT{full};
FixMinInport: INPORT OF FixMinCall {Full};
FixMinOutport: OUTPORT OF FixMinInport;

-- table driving the two passes:
FixFormals: ordered table of FixFullOutport {full};

end definitions
