-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- SCCS Info: @(#)cload.d	1.5 3/13/90

-- fake hermes processes written in C and built into the interpreter.  the
-- file cfunc/cproctab.c contains the list of loadable C processes.  the
-- hermes interfaces to them are defined in various modules in the definitions
-- directory.

CLoad: using (common) definitions

  CLoad: callmessage (
    name: charstring,
    capability: polymorph
  ) constant (name)
    exit { full }
    exception badName { init(name) };

  CLoadQ: inport of CLoad { init(name) };
  CLoadFn: outport of CLoadQ;

end definitions
