/* Filename: TtyTextP.h */
/* Author: Andy Lowry */
/* SCCS Info: @(#)TtyTextP.h	1.1 6/18/90 */

#ifndef _TtyTextP_h
#define _TtyTextP_h

#include "TtyText.h"
#include <X11/TextP.h>

extern XtActionsRec textActionsTable[];
extern Cardinal textActionsTableCount;

typedef struct {int empty;} TtyTextClassPart;

typedef struct _TtyTextClassRec {
    CoreClassPart	core_class;
    SimpleClassPart	simple_class;
    TextClassPart	text_class;
    TtyTextClassPart    tty_text_class;
} TtyTextClassRec;

extern TtyTextClassRec ttyTextClassRec;

typedef struct {
    XtCallbackList callback;
} TtyTextPart;

typedef struct _TtyTextRec {
    CorePart		core;
    SimplePart		simple;
    TextPart		text;
    TtyTextPart 	tty_text;
} TtyTextRec;

#endif  _TtyTextP_h
