/* (C) Copyright International Business Machines Corporation 23 January */
/* 1990.  All Rights Reserved. */
/*  */
/* See the file USERAGREEMENT distributed with this software for full */
/* terms and conditions of use. */
/* File: unixenv.h */
/* Author: David F. Bacon */
/* SCCS Info: @(#)unixenv.h	1.5 4/20/90 */

#define PATH_VARNAME "HPATH"
#define LOADPATH_VARNAME "HLOADPATH"
#define LINKPATH_VARNAME "HLINKPATH"

#define ABSPROG_OBJECT_EXTENSION ".ao"
#define PROC_OBJECT_EXTENSION ".po"
#define DEF_OBJECT_EXTENSION  ".do"
