/* (C) Copyright International Business Machines Corporation 23 January */
/* 1990.  All Rights Reserved. */
/*  */
/* See the file USERAGREEMENT distributed with this software for full */
/* terms and conditions of use. */
/* File: stubs.c */
/* Author: Andy Lowry */
/* SCCS Info: @(#)stubs.c	1.3 3/13/90 */

/* stubs for routines required for linking but not really applicable */
/* to the assembler (necessary because we share libraries with other */
/* executables */

int
get_object_talloc_size()	/* called by storage_init */
{
  return(sizeof(char*));	/* need something nonzero and at least */
				/* large enough to hold a pointer so */
				/* quickcells can be linked together */
}
