-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- File: fixdefs.d
-- Author: Rob Strom
-- SCCS Info: @(#)fixdefs.d	1.3 3/13/90

-- definitions local to fixdefs.p
fixdefs: USING(stdenv, filedef) 
definitions
-- Procedure to put init(comp)
-- into Minimum TS provided it's in exit TS
-- and to do the same with init record components
FixdefsCall: CALLMESSAGE (
  Std: Stdenv, -- standard environment
  ToBeFixed: filed_definition, -- definition being fixed
  Definitions: definitions_modules -- all the definitions needed to resolve types
  )
CONSTANT(Std) 
EXIT{full}
MINIMUM{full};
FixdefsInport: INPORT OF FixdefsCall {Full};
FixdefsOutport: OUTPORT OF FixdefsInport;


end definitions
