-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- File: programcacheimp.d
-- Author: David F. Bacon
-- SCCS Info: @(#)programcacheimp.d	1.3 3/13/90

programCacheImp: using (unix) definitions

  Cache: table of CacheEntry { full } keys (filename);

  CacheEntry: record (
      filename: charstring,	/* full name of file */
      mtime: long,		/* time last modified (for cache invalidate) */
      object: program		/* the object being cached */
  );

end definitions
