-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- File: programcache.d
-- Author: David F. Bacon
-- SCCS Info: @(#)programcache.d	1.3 3/13/90

programCache: using (load, unixStat)
definitions

  initCache: callmessage (
    read: load_func,
    write: storeFunc,
    stat: statMtimeFunc
  ) exit { full };

  initCacheQ: inport of initCache { full };
  initCacheFunc: outport of initCacheQ;

end definitions
