-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
plumberInt: using (plumbing, rmanager) definitions
    
    procInfo: record (
	id: procId,
	rm: rManager
      );
    
    procInfoSet: table of procInfo { full } keys (id);
    
  end definitions

