-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- makePipe is the interface to processes that create pipelines with which
-- processes can be connected.  for instance, the makePipeChar process returns
-- an outport which accepts chars and the inport to which it is bound.  the 
-- out end is then published in the resource manager of the source, and the in
-- end in the resource manager of the sink.

makePipe: using (common) definitions
    
    makePipe: callmessage (	-- returns an object pair, usually an outport
				-- bound to an inport.
	output: polymorph,	  
	input: polymorph
      )
    exit { full };
    
    makePipeQ: inport of makePipe { };
    makePipeFn: outport of makePipeQ;
    
  end definitions

