# (C) Copyright International Business Machines Corporation 23 January 
# 1990.  All Rights Reserved. 
#  
# See the file USERAGREEMENT distributed with this software for full 
# terms and conditions of use. 
# SCCS Info: @(#)list.awk	1.2 3/13/90

/^[a-z_]*/ {
  print $1 "_list : '(' " $1 "_list1 ')' ;"
  print $1 "_list1 : " $1 "_list2 ;"
  print $1 "_list1 : empty ;"
  print $1 "_list2 : " $1 "_list2 ',' " $1 " ;"
  print $1 "_list2 : " $1 " ;"
  print ""
}
