/* (C) Copyright International Business Machines Corporation 23 January */
/* 1990.  All Rights Reserved. */
/*  */
/* See the file USERAGREEMENT distributed with this software for full */
/* terms and conditions of use. */
/* File: release.h */
/* Author: Andy Lowry */
/* SCCS Info: @(#)release.h	1.2 3/13/90 */

/* This file identifies the Hermes release number from which a given */
/* interpreter was built.  The release number is filled in by the */
/* release utilities when a release is built.  If the release */
/* identifier is "UNKNOWN", then this interpreter was not part of an */
/* official Hermes release, or the release was not properly prepared. */

static char hermes_release[] = "Hermes Release: 0.6alpha on Jun 20, 1990";
