/* Filename: TtySrc.h */
/* Author: Andy Lowry */
/* SCCS ID: @(#)TtySrc.h	1.1 6/18/90 */

/* Public definitions for Tty Source for text widgets */

#define XtNmaxLines	"maxLines"
#define XtCMaxLines	"MaxLines"

extern Boolean TtyAccumulateTypein(); /* src, flag */
    /* XtTextSource src;	*/
    /* Boolean      flag;	*/

extern XtTextSource XtTtySourceCreate(); /* widget, args, num_args */
    /* Widget widget;		*/
    /* Arglist args;		*/
    /* Cardinal num_args;	*/
