/* (C) Copyright International Business Machines Corporation 23 January */
/* 1990.  All Rights Reserved. */
/*  */
/* See the file USERAGREEMENT distributed with this software for full */
/* terms and conditions of use. */
/* File: avltree.h */
/* Author: David F. Bacon */
/* SCCS Info: @(#)avltree.h	1.2 3/13/90 */

typedef struct avlnode_struct {
    struct avlnode_struct *left, *right;
    valcell value;
    short int balance;		/* -1 means leaning left, +1 means right */
} avlnode_t;
