-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- File: const.def
-- Author: Andy Lowry
-- SCCS Info: @(#)const.def	1.2 3/13/90

-- Some standard symbol definitions for LI source modules

#define PRIMARY_KEY 0	
#define SECONDARY_KEY 1
#define COMPONENT 0
#define LI_PROGRAM 3
#define Initport 0
#define load(obj,file,name) \
     chs_lit file, name \
     read obj file 
