-- (C) Copyright International Business Machines Corporation 23 January 
-- 1990.  All Rights Reserved. 
--  
-- See the file USERAGREEMENT distributed with this software for full 
-- terms and conditions of use. 
-- File: unixstat.d
-- Author: David F. Bacon
-- SCCS Info: @(#)unixstat.d	1.3 3/13/90

unixStat: using (predefined, unix) definitions

  statMtime: callmessage (
    filename: charstring,
    mtime: long
  ) constant (filename)
    exit { full };

  statMtimeQ: inport of statMtime { init(filename) };
  statMtimeFunc: outport of statMtimeQ;

end definitions
